import 'package:state_graph_bloc/state_graph_bloc.dart';

import 'dummy_event.dart';
import 'dummy_state.dart';

class DummyBloc extends StateGraphBloc<DummyEvent, DummyState> {
  final Function(
          DummyState previousState, DummyEvent event, DummyState newState)
      onTransitionCallback;

  DummyBloc({this.onTransitionCallback}) : super(StateInitialising());

  @override
  void onTransition(
      DummyState previousState, DummyEvent event, DummyState newState) {
    super.onTransition(previousState, event, newState);
    onTransitionCallback?.call(previousState, event, newState);
  }

  @override
  StateGraph<DummyEvent, DummyState> buildGraph() =>
      StateGraph<DummyEvent, DummyState>(
        {
          StateInitialising: {
            LoadEvent: transitionWithSideEffect(
              (state, event) => StateLoading(),
              (state, event) {
                // There should be some fetching
                add(LoadingCompletedEvent());
              },
            ),
          },
          StateLoading: {
            LoadingCompletedEvent: transition((state, event) => StateReady(0)),
          },
          StateReady: {
            IncrementEvent: transition(
              (StateReady state, event) => StateReady(state.counter + 1),
            ),
            // Test global event override.
            RestartEvent: transition((state, event) => StateReady(0)),
          },
        },
        {
          RestartEvent: transition((state, event) {
            return StateInitialising();
          })
        },
      );

  Stream<bool> isLoading() => bindState((state) {
        if (state is StateLoading) {
          return true;
        } else {
          return false;
        }
      });
}
